BINDING_HEADER_BIGGERBATTLESHOUT = "BigBattleShout";

function BBS_ShouldShout() return shouldShout end

local _, playerClass = UnitClass("player");
local playerName = UnitName("player");
local tooltip = CreateFrame("GameTooltip", "BBSTooltip", nil, "GameTooltipTemplate");
local BBS_Frame = CreateFrame("Frame");
local BBSEvents = {}


local ShoutAP = {
  [11551] = 193,
  [25289] = 232,
  [2048] = 306,
  [47436] = 550,
};
local BBSrank;
local BBSap;

local BBSTicker;

local isInCombat = false;
local ICDTimestamp = 0;
function BBS_SetICD(time)
  ICDTimestamp = time;
end
function BBS_GetICD() return ICDTimestamp end

local shouldShout = false;

function BBS_Initialize()
  C_Timer.NewTicker(2, function()
    BBS_CalculateBaseAP();
    BBS_SetWrathPieces();
  end);
end

function BBS_CalculateBaseAP()
  if(not BBSrank) then BBS_CreateTooltip() end
  local _ , _, _, _, currentRank, maxRank, _, _ = GetTalentInfo(2, 8);
  BBSap = math.floor(ShoutAP[BBSrank] * (1 + 0.05 * currentRank)) + 28;
end

function BBS_CreateTooltip()
  local _, _, _, _, _, _, rank = GetSpellInfo("Battle Shout");
  BBSrank = rank;
  tooltip:SetSpellByID(BBSrank);
  tooltip:SetOwner(WorldFrame, "ANCHOR_NONE");
end

function BBS_GetTooltipAP()
  if(not BBSrank) then BBS_CreateTooltip() end
  tooltip:SetSpellByID(BBSrank);
  local costText = BBSTooltipTextLeft4:GetText();
  if(costText) then
    return string.match(costText, "yards by ([0-9]+)");
  else
    return tooltipAPLast;
  end
end

function BBS_QueueSwap(ignoreICD)
  if(BBS_GetICD() < GetTime() or ignoreICD) then
    if(BBS_HasSwapQueued()) then
      BBS_SetQueueStatus(false);
      print("BAA: Gear queue canceled.");
      return;
    end
    if(isInCombat) then
      BBS_SetQueueStatus(true);
      print("BAA: Gear swap queued.");
    else
      BBS_EquipQueued();
    end
    BBS_SetICD(GetTime() + 0.8);
  end
end

local tooltipAPLast = 0;
function BBS_ToggleTicker(state)
  if(state) then
    BBSTicker = C_Timer.NewTicker(0.05, function()
      if(not isInCombat) then
        local tooltipAP = BBS_GetTooltipAP();
        if(tooltipAP ~= nil) then
          tooltipAPLast = tonumber(tooltipAP);
          if(tooltipAPLast > BBSap) then
            print("BAA: Set Bonus Active.");
            shouldShout = true;
            tooltipAPLast = 0;
            BBSTicker:Cancel();
          end
        end
      end
    end);
  else
    if(BBSTicker ~= nil) then
      BBSTicker:Cancel();
    end
  end
end

function BBSEvents:COMBAT_LOG_EVENT_UNFILTERED(...)
  local _, eventType, _, sourceGUID,
      sourceName, _, _, destGUID,
      destName, _, _, spellId,
      spellName, _, auraType, _,
      _,isCrit,_ = CombatLogGetCurrentEventInfo();
  if(eventType == "SPELL_CAST_SUCCESS") then
    if(spellName == "Battle Shout" and sourceName == playerName and shouldShout) then
      shouldShout = false;
      BBS_QueueSwap(true);
    end
  end
end

function BBSEvents:PLAYER_REGEN_ENABLED(...)
  isInCombat = false;
  if(BBS_HasSwapQueued()) then
    BBS_EquipQueued();
  end
end

function BBSEvents:PLAYER_REGEN_DISABLED(...)
  isInCombat = true;
end

function BBSEvents:CHARACTER_POINTS_CHANGED(...)
  BBS_CalculateBaseAP();
end

local BagTicker;
local lastBagUpdate = 0;
function BBSEvents:BAG_UPDATE(...)
  lastBagUpdate = GetTime();
  if(BagTicker == nil) then
    BagTicker = C_Timer.NewTicker(2, function()
      if(not isInCombat and GetTime() - lastBagUpdate > 5) then
        BBS_SetWrathPieces();
        BagTicker:Cancel();
      end
    end);
  end
end

if(playerClass ~= "WARRIOR" or UnitLevel("player") < 60) then
  return
else
  BBS_Initialize();
  BBS_Frame:SetScript("OnEvent", function(self, event, ...)
    BBSEvents[event](self, ...);
  end);
  for k, v in pairs(BBSEvents) do
    BBS_Frame:RegisterEvent(k);
  end
end

SLASH_BBS1 = "/bbs"
SLASH_BBS2 = "/bigbattleshout"
SlashCmdList["BBS"] = function() BBS_IncrementTrinketSetting(); end
